/*
 * Author: Jayesh Patel
 *
 * Copyright (c) 2016 Trustwave Holdings, Inc.
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Trustwave Holdings, Inc.  Use of this software is governed by
 * the terms and conditions of the license statement and limited
 * warranty furnished with the software.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD TRUSTWAVE HOLDINGS INC.,
 * ITS RELATED COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST
 * ANY CLAIMS OR LIABILITIES ARISING OUT OF OR RESULTING FROM THE USE,
 * MODIFICATION, OR DISTRIBUTION OF PROGRAMS OR FILES CREATED FROM,
 * BASED ON, AND/OR DERIVED FROM THIS SOURCE CODE FILE.
 */
library swg_formatter_util;


import 'package:swg_ui_common/src/model/action.dart';
import 'package:swg_ui_common/src/model/certificate_groups.dart';
import 'package:swg_ui_common/src/model/identifier.dart';
import 'package:swg_ui_common/src/model/inheritance.dart';
import 'package:swg_ui_common/src/model/policy_result.dart';
import 'package:swg_ui_common/src/model/rule.dart';
import 'package:swg_ui_common/src/model/rule_type.dart';
import 'package:swg_ui_common/src/model/url_category.dart';

class FormatterUtil {

  final String LOCK_ICON = "<i class='icon tw tw-lock blackTxt'></i>";
  final String UNLOCK_ICON = "<i class='icon tw tw-unlock greenTxt'></i>";

  static String getCustomFormattedString(FormatterType formatter, dynamic value, [String color]) {
    String displayString = "";
    if (value == null || formatter == null) return displayString;
    return _getDisplayValue(formatter, value, color);
  }

  static Map _prohibitOverrideIcons = {
    true : "<i class='icon tw tw-lock blackTxt'></i>",
    false : "<i class='icon tw tw-unlock greenTxt'></i>"
  };

  static Map _prohibitOverrideToolTipMap = {
    true : "Override Prohibited In Child Policies",
    false : "Override Allowed In Child Policies"
  };

  static Map _ruleTypeDisplayStrings = {
    RuleType.APPLICATION_CONTROL: "Application Control",
    RuleType.CONTENT_SIZE: "Content Size",
    RuleType.DLP: "DLP",
    RuleType.FILE_EXTENSION: "File Extension",
    RuleType.FILE_TYPE: "File Type",
    RuleType.URL_CAT: "URL Category",
    RuleType.URL_LIST: "URL List",
    RuleType.TIME_FRAME: "Timeframe",
    RuleType.AUTHENTICATION_METHODS: "Authentication Method"
  };

  static Map _ruleTypesDisplayStrings = {
    RuleTypes.APP_CONTROL_PROFILE: "Application Control Profile",
    RuleTypes.CONTENT_SIZE: "Content Size",
    RuleTypes.CUSTOM_APP_CONTROL: "Custom Application Control",
    RuleTypes.CUSTOM_URL: "Custom URL",
    RuleTypes.DLP: "DLP",
    RuleTypes.FILE_EXTENSION: "File Extension",
    RuleTypes.FILE_EXTENSION_GROUP: "File Extension Group",
    RuleTypes.FILE_TYPE: "File Type",
    RuleTypes.URL_CATEGORY: "URL Category",
    RuleTypes.URL_LIST: "URL List",
    RuleTypes.USER_AGENT: "User Agent",
    RuleTypes.QUOTA_PROFILE: "Quota"
  };

  static Map _policyTypeDisplayStrings = {
    PolicyTypes.NAMED: "Named",
    PolicyTypes.IDENTITY_SPECIFIC_RULES: "Identity Specific Rules",
    PolicyTypes.OOO: "Out Of Office"
  };

  static Map _identityTypeDisplayStrings = {
    IdentityType.USER: "User",
    IdentityType.GROUP: "Group",
    IdentityType.IP: "Location"
  };

  static Map _transactionActionDisplayStrings = {
    TransactionAction.ALLOW: "Allowed",
    TransactionAction.BLOCK: "Blocked",
    TransactionAction.WHITELIST : "Whitelist",
    TransactionAction.COACH : "Coach",
    TransactionAction.XRAY : "X-Ray",
    TransactionAction.HTTPS_BYPASS : "Https Bypass",
    TransactionAction.HTTPS_BLOCK : "Https Block"
  };

  static Map _transactionActionPanelIcons = {
    TransactionAction.ALLOW: "tw-check",
    TransactionAction.WHITELIST: "tw-check",
    TransactionAction.XRAY: "tw-check",
    TransactionAction.BLOCK: "tw-ban",
    TransactionAction.COACH: "tw-check",
    TransactionAction.HTTPS_BYPASS : "tw-check",
    TransactionAction.HTTPS_BLOCK : "tw-ban"
  };

  static Map _transactionActionPanelIconClasses = {
    TransactionAction.ALLOW: "greenTxt",
    TransactionAction.WHITELIST: "greenTxt",
    TransactionAction.XRAY: "greenTxt",
    TransactionAction.BLOCK: "redTxt",
    TransactionAction.COACH: "greenTxt",
    TransactionAction.HTTPS_BYPASS : "greenTxt",
    TransactionAction.HTTPS_BLOCK : "redTxt"
  };

  static Map _inheritanceIconMap = {
    Inheritance.INHERITED: "<i class='icon tw tw-user blueTxt'></i>",
    Inheritance.LOCKED_FOR_OVERRIDE: "<i class='icon tw tw-lock blackTxt'></i>",
    Inheritance.NOT_INHERITED: "",
    Inheritance.OVERRIDING: "<i class='icon tw tw-user-plus greenTxt'></i>",
    "INHERITED": "<i class='icon tw tw-user blueTxt'></i>",
    "LOCKED_FOR_OVERRIDE": "<i class='icon tw tw-lock blackTxt'></i>",
    "NOT_INHERITED": "",
    "OVERRIDING": "<i class='icon tw tw-user-plus greenTxt'></i>"
  };

  static Map _inheritanceToolTipMap = {
    Inheritance.INHERITED: "Inherited From Parent Policy",
    Inheritance.LOCKED_FOR_OVERRIDE: "Locked For Override By Parent Policy",
    Inheritance.NOT_INHERITED: "Rule Defined In This Policy",
    Inheritance.OVERRIDING: "Overriding Rule In Parent Policy",
    "INHERITED": "Inherited From Parent Policy",
    "LOCKED_FOR_OVERRIDE": "Locked For Override By Parent Policy",
    "NOT_INHERITED": "Rule Defined In This Policy",
    "OVERRIDING": "Overriding Rule In Parent Policy"
  };

  static Map _scanTypeDisplayStrings = {
    ScanType.REQUEST: "Request",
    ScanType.RESPONSE: "Response",
    ScanType.RESPONSE_INNER: "Response/Inner",
    "REQUEST": "Request",
    "RESPONSE": "Response",
    "RESPONSE_INNER": "Response/Inner"
  };

  static Map _eventTypesDisplayStrings = {
    EventTypes.REDIRECT: "Redirect",
    EventTypes.COACH_PAGE: "Coach",
  };

  static Map _categoryInheritedIcons = {
    "truetrue": "<i class='icon tw tw-user blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "truefalse": "<i class='icon tw tw-user blackTxt' style='padding: 1px;font-size: 16px'></i>"
  };

  static Map _categoryInheritedTooltips = {
    "truetrue": "Inherited Custom",
    "truefalse": "Inherited"
  };

  static Map _customCategoryExistsTooltips = {
    true: "Custom Categories Exist",
    false: ""
  };

  static Map _rulesExistsTooltips = {
    true: "Rules Exist",
    false: ""
  };

  static Map _actionDisplayString = {
    Action.WHITELIST: "Whitelist",
    Action.ALLOW: "Allow",
    Action.BLOCK: "Block",
    Action.COACH: "Coach",
    Action.XRAY: "X-Ray",
    Action.PASS: "Pass"
  };

  static Map _cerGroups = {
    CerGroup.CRL :"CLR",
    CerGroup.TRUSTED_ROOT_CA :"Trusted Root CA",
    CerGroup.UNTRUSTED_PUBLISHERS :"Untrusted Publishers"
  };

  /*static Map _inheritanceOverrideActionIcons = {
    "Action.WHITELIST": "<i class='icon tw tw-user blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.ALLOW": "<i class='icon tw tw-check greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.BLOCK": "<i class='icon tw tw-ban redTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.COACH": "<i class='icon tw tw-ambulance blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.XRAY": "<i class='icon tw tw-search blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.PASS": "<i class='icon tw tw-check-circle greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Whitelist": "<i class='icon tw tw-user blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Allow": "<i class='icon tw tw-check greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Block": "<i class='icon tw tw-ban redTxt' style='padding: 1px;font-size: 16px'></i>",
    "Coach": "<i class='icon tw tw-ambulance blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "X-ray": "<i class='icon tw tw-search blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Pass": "<i class='icon tw tw-check-circle  greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "multiple": "<i class='icon tw tw-clock-o blueTxt' style='padding: 1px;font-size: 16px'></i>"
  };*/

  static Map _inheritanceOverrideActionIcons = {
    "Action.ALLOW": "<i class='icon tw tw-check greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.ALLOW_grayTxt": "<i class='icon tw tw-check grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.BLOCK": "<i class='icon tw tw-ban redTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.BLOCK_grayTxt": "<i class='icon tw tw-ban grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.COACH": "<i class='icon tw tw-ambulance blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.COACH_grayTxt": "<i class='icon tw tw-ambulance grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.XRAY": "<i class='icon tw tw-search blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.XRAY_grayTxt": "<i class='icon tw tw-search grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.PASS": "<i class='icon tw tw-check-circle greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Action.PASS_grayTxt": "<i class='icon tw tw-check-circle greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Allow": "<i class='icon tw tw-check greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Allow_grayTxt": "<i class='icon tw tw-check grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Block": "<i class='icon tw tw-ban redTxt' style='padding: 1px;font-size: 16px'></i>",
    "Block_grayTxt": "<i class='icon tw tw-ban grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Coach": "<i class='icon tw tw-ambulance blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "Coach_grayTxt": "<i class='icon tw tw-ambulance grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "X-ray": "<i class='icon tw tw-search blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "X-ray_grayTxt": "<i class='icon tw tw-search grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "Pass": "<i class='icon tw tw-check-circle greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "Pass_grayTxt": "<i class='icon tw tw-check-circle greenTxt' style='padding: 1px;font-size: 16px'></i>",
    "multiple": "<i class='icon tw tw-clock-o blueTxt' style='padding: 1px;font-size: 16px'></i>",
    "multiple_grayTxt": "<i class='icon tw tw-clock-o grayTxt' style='padding: 1px;font-size: 16px'></i>",
    "locked": "<i class='icon tw tw-lock blackTxt' style='padding: 1px;font-size: 16px'></i>",
    "locked_blackTxt": "<i class='icon tw tw-lock blackTxt' style='padding: 1px;font-size: 16px'></i>"
  };

  static Map _inheritanceOverrideActionIconToolTips = {
    "Action.ALLOW": "Allow",
    "Action.ALLOW_grayTxt": "Allow",
    "Action.BLOCK": "Block",
    "Action.BLOCK_grayTxt": "Block",
    "Action.COACH": "Coach",
    "Action.COACH_grayTxt": "Coach",
    "Action.XRAY": "X-ray",
    "Action.XRAY_grayTxt": "X-ray",
    "Action.PASS": "Pass",
    "Action.PASS_grayTxt": "Pass",
    "Allow": "Allow",
    "Allow_grayTxt": "Allow",
    "Block": "Block",
    "Block_grayTxt": "Block",
    "Coach": "Coach",
    "Coach_grayTxt": "Coach",
    "X-ray": "X-ray",
    "X-ray_grayTxt": "X-ray",
    "Pass": "Pass",
    "Pass_grayTxt": "Pass",
    "multiple": "Time Dependent",
    "multiple_grayTxt": "Time Dependent",
    "locked": "Locked For Override By Parent Policy",
    "locked_grayTxt": "Locked For Override By Parent Policy"
  };

  static String _getDisplayValue(FormatterType formatter, dynamic value, [String color]) {
    String displayString = "";
    if (formatter == FormatterType.IDENTITY) {
      displayString = _identityTypeDisplayStrings[value];
    } else if (formatter == FormatterType.POLICY_TYPES) {
      displayString = _policyTypeDisplayStrings[value];
    } else if (formatter == FormatterType.RULE_TYPE) {
      displayString = _ruleTypeDisplayStrings[value];
    } else if (formatter == FormatterType.RULE_TYPES) {
      if (value is Rule) {
        displayString = _ruleTypesDisplayStrings[value.type];
      } else {
        displayString = _ruleTypesDisplayStrings[value];
      }
    } else if (formatter == FormatterType.INHERITANCE_ICON) {
      if (value is Rule) {
        displayString = _inheritanceIconMap[value.inherited];
      } else {
        displayString = _inheritanceIconMap[value];
      }
    } else if (formatter == FormatterType.INHERITANCE_TOOLTIP) {
      if (value is Rule) {
        displayString = _inheritanceToolTipMap[value.inherited];
      } else {
        displayString = _inheritanceToolTipMap[value];
      }
    } else if (formatter == FormatterType.TRANSACTION_ACTION) {
      displayString = _transactionActionDisplayStrings[value];
    } else if (formatter == FormatterType.TRANSACTION_ACTION_ICON) {
      displayString = _transactionActionPanelIcons[value];
    } else if (formatter == FormatterType.TRANSACTION_ACTION_ICON_CLASS) {
      displayString = _transactionActionPanelIconClasses[value];
    } else if (formatter == FormatterType.PROHIBIT_OVERRIDE) {
      if (value is Rule) {
        displayString = _prohibitOverrideIcons[value.prohibitOverride];
      } else {
        displayString = _prohibitOverrideIcons[value];
      }
    } else if (formatter == FormatterType.PROHIBIT_OVERRIDE_TOOLTIP) {
      if (value is Rule) {
        displayString = _prohibitOverrideToolTipMap[value.prohibitOverride];
      } else {
        displayString = _prohibitOverrideToolTipMap[value];
      }
    } else if (formatter == FormatterType.SCAN_TYPE) {
      displayString = _scanTypeDisplayStrings[value];
    } else if (formatter == FormatterType.EVENT_TYPES) {
      displayString = _eventTypesDisplayStrings[value];
    } else if (formatter == FormatterType.ACTION) {
      if (value is Action) {
        displayString = _actionDisplayString[value];
      } else {
        if (value.action != null) {
          displayString = _actionDisplayString[value.action];
        }
      }
    } else if (formatter == FormatterType.TIMEFRAME_IDENTIFIER) {
      if (value is Identifier) {
        displayString = value.name;
      } else {
        if (value.timeframeIdentifier != null) {
          displayString = value.timeframeIdentifier.name;
        }
      }
    } else if (formatter == FormatterType.EUM_IDENTIFIER) {
      if (value is Identifier) {
        displayString = value.name;
      } else {
        if (value.endUserMessageIdentifier != null) {
          displayString = value.endUserMessageIdentifier.name;
        }
      }
    } else if (formatter == FormatterType.POLICY_NAME) {
      if (value is PolicyDetails) {
        displayString = value.name;
      }
    } else if(formatter == FormatterType.CERTIFICATE_GROUP){
      if( value is CerGroup ) {
        displayString = _cerGroups[value];
      }
    } /*else if(formatter == FormatterType.INHERITANCE_ACTION_ICON) {
      displayString = _inheritanceOverrideActionIcons[value];
    } else if(formatter == FormatterType.OVERRIDE_ACTION_ICON) {
      displayString = _inheritanceOverrideActionIcons[value];
    }*/ else if(formatter == FormatterType.INHERITANCE_ACTION_ICON) {
      String key = "";
      if (color != null && color.isNotEmpty) {
        key = value + "_" + color;
      } else {
        key = value;
      }
      displayString = _inheritanceOverrideActionIcons[key];
    } else if(formatter == FormatterType.OVERRIDE_ACTION_ICON) {
      String key = "";
      if (color != null && color.isNotEmpty) {
        key = value + "_" + color;
      } else {
        key = value;
      }
      displayString = _inheritanceOverrideActionIcons[key];
    } else if(formatter == FormatterType.INHERITANCE_ACTION_ICON_TOOLTIP) {
      displayString = _inheritanceOverrideActionIconToolTips[value];
    } else if(formatter == FormatterType.OVERRIDE_ACTION_ICON_TOOLTIP) {
      displayString = _inheritanceOverrideActionIconToolTips[value];
    } else if(formatter == FormatterType.CATEGORY_INHERITED) {
      if (value is UrlCategory) {
        displayString = _categoryInheritedIcons[value.inheritedCustom];
        if (displayString == null) {
          displayString = "";
        }
      }
    } else if(formatter == FormatterType.CATEGORY_INHERITED_TOOLTIP) {
      displayString = _categoryInheritedTooltips[value];
    } else if (formatter == FormatterType.CUSTOM_CATEGORY_EXISTS_TOOLTIP) {
      displayString = _customCategoryExistsTooltips[value];
    } else if (formatter == FormatterType.RULES_EXISTS_TOOLTIP) {
      displayString = _rulesExistsTooltips[value];
    }

    return displayString;
  }
}

enum FormatterType {
  IDENTITY,
  POLICY_TYPES,
  RULE_TYPE,
  RULE_TYPES,
  TRANSACTION_ACTION,
  TRANSACTION_ACTION_ICON,
  TRANSACTION_ACTION_ICON_CLASS,
  INHERITANCE_ICON,
  INHERITANCE_TOOLTIP,
  PROHIBIT_OVERRIDE,
  PROHIBIT_OVERRIDE_TOOLTIP,
  SCAN_TYPE,
  EVENT_TYPES,
  ACTION,
  TIMEFRAME_IDENTIFIER,
  EUM_IDENTIFIER,
  POLICY_NAME,
  CERTIFICATE_GROUP,
  INHERITANCE_ACTION_ICON,
  OVERRIDE_ACTION_ICON,
  INHERITANCE_ACTION_ICON_TOOLTIP,
  OVERRIDE_ACTION_ICON_TOOLTIP,
  CATEGORY_INHERITED,
  CATEGORY_INHERITED_TOOLTIP,
  CUSTOM_CATEGORY_EXISTS_TOOLTIP,
  RULES_EXISTS_TOOLTIP
}