library swg.config.ui.angular.component.app_profiles;

import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:angular_components/model/selection/selection_model.dart';
import 'package:angular_components/model/selection/selection_options.dart';
import 'package:swg_config_ui_angular/src/common/formatter_util.dart';
import 'package:swg_config_ui_angular/src/common/locale.dart';
import 'package:swg_ui_common/src/model/action.dart';
import 'package:swg_ui_common/src/model/app_control_item.dart';
import 'package:swg_ui_common/src/model/app_control_item_action.dart';
import 'package:swg_ui_common/src/model/app_control_profile.dart';
import 'package:swg_ui_common/src/model/app_profile_data.dart';
import 'package:swg_ui_common/src/model/change_action.dart';
import 'package:swg_ui_common/src/service/building_block_service.dart';
import 'package:swg_ui_common/src/util/customer_model_locator.dart';
import 'package:tw_portal/portal.dart';
import 'package:tw_ui_angular/tw_ui_angular.dart';

// ignore: uri_has_not_been_generated
import 'action_custom_renderer.template.dart' as action_custom;

@Component(selector: 'twa-swg-app-profiles', templateUrl: 'app_profiles.html', styleUrls: const [
  'app_profiles.css'
], pipes: const [
  InterpretPipe
], exports: const [
  SwgConfigLocaleInfo,
  ChangeAction,
  FormatType
], directives: const [
  CORE_DIRECTIVES,
  twaDirectives
], providers: const [
  const Provider(ComponentService, useFactory: getComponentService),
  const Provider(ComponentBase, useExisting: AppProfilesComponent)
])
class AppProfilesComponent extends PortalComponent {
  ///
  /// Services & Constructor
  ///

  final Logger _log = new Logger("AppProfilesComponent");

  @override
  final List<String> localeRoots = [
    SwgConfigLocaleInfo.PORTAL_KEY,
    SwgConfigLocaleInfo.SWG_CONFIG_KEY,
    SwgConfigLocaleInfo.SWG_CONFIG_COMMON_KEY
  ];

  final BuildingBlockService _buildingBlockService;

  AppProfilesComponent(ComponentService cmpSvc, this._buildingBlockService) : super(cmpSvc);

  ///
  /// Template Members
  ///

  final int nameLength = 100;

  bool waiting = false;
  bool rightPanelOpened = false;
  List<AppControlProfile> appProfiles;
  AppControlProfile selectedAppProfile;
  ChangeAction operation;
  int selectedIndex;
  String formHeaderValue = "";
  List<AppControlItemAction> controlItemGridProvider;
  bool controlItemAddEditDialogVisible = false;
  String dialogLabel;
  ChangeAction controlItemAction;
  AppControlItemAction selectedControlItem;
  bool showInUseAlert = false;
  bool controlItemFormValid = true;

  String get controlItemFormPrimaryState =>
      controlItemFormValid ? ButtonComponent.NORMAL_STATE : ButtonComponent.ERROR_STATE;
  String controlItemFormValidationMessage = "";
  ComponentFactory actionCustomFactory = action_custom.ActionCustomRendererComponentNgFactory;

  ///
  /// Template Members: Action Dropdown
  ///

  List<Action> actionsDataProvider;
  SelectionModel<Action> actionSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);

  SelectionOptions<Action> get actions => new SelectionOptions.fromList(actionsDataProvider ?? []);
  ItemRenderer<Action> actionItemRenderer = new CachingItemRenderer<Action>((action) => "${FormatterUtil
      .getCustomFormattedString(FormatterType.ACTION, action)}");

  String get actionSelectLabel => actionSelectionModel.selectedValues.length > 0
      ? actionItemRenderer(actionSelectionModel.selectedValues.first)
      : "";

  Action get selectedAction =>
      actionSelectionModel.selectedValues.length > 0 ? actionSelectionModel.selectedValues.first : "";

  ///
  /// Template Members: Application Dropdown
  ///

  List<String> applicationsDataProvider;
  Map _appControlItems = {};

  SelectionModel<String> applicationSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);

  SelectionOptions<String> get applications => new SelectionOptions.fromList(applicationsDataProvider ?? []);
  ItemRenderer<String> applicationItemRenderer = new CachingItemRenderer<String>((application) => "${application}");

  String get applicationSelectLabel => applicationSelectionModel.selectedValues.length > 0
      ? applicationItemRenderer(applicationSelectionModel.selectedValues.first)
      : "";

  ///
  /// Template Members: Item Dropdown
  ///

  List applicationItems = [];
  bool itemDropdownDisabled = true;

  SelectionModel<String> itemSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);

  SelectionOptions<String> get items => new SelectionOptions.fromList(applicationItems ?? []);
  ItemRenderer<String> itemItemRenderer = new CachingItemRenderer<String>((item) => "${item}");

  String get itemSelectLabel =>
      itemSelectionModel.selectedValues.length > 0 ? itemItemRenderer(itemSelectionModel.selectedValues.first) : "";

  ///
  /// Template Members: DOM element
  ///

  @ViewChild('appProfilesGrid')
  DatagridComponent appProfilesGrid;

  @ViewChild('controlItemAddEditDialog')
  DialogComponent controlItemAddEditDialog;

  @ViewChild('controlItemAddEditForm')
  FormComponent controlItemAddEditForm;

  @ViewChild('controlItemGrid')
  DatagridComponent controlItemGrid;

  @ViewChild('fileInput')
  var fileInput;

  ///
  /// Template Members: Event Handling
  ///

  onAdd(DatagridAddEvent addEvent) {
    _log.debug("onAdd addEvent $addEvent");
    _resetFormValues();
    formHeaderValue = interpret(SwgConfigLocaleInfo.ADD_APP_PROFILE);
    selectedAppProfile = new AppControlProfile();
    operation = ChangeAction.ADD;
    rightPanelOpened = true;
    controlItemGridProvider = [];
  }

  onDelete(DatagridDeleteEvent deleteEvent) {
    _log.debug("onDelete deleteEvent $deleteEvent");

    if (selectedAppProfile.inUse) {
      showInUseAlert = true;
      return;
    }

    var deleteItem = deleteEvent.selectedItem;
    appProfiles.remove(deleteItem);
    _resetFormValues();
    _resetDetailViewValues();
  }

  onRefresh(DatagridRefreshEvent refreshEvent) {
    _log.debug("onRefresh refreshEvent $refreshEvent");
    _resetFormValues();
    _resetDetailViewValues();
    _getAppProfilesForCustomer();
  }

  onSelectItem(AppControlProfile selectedItem) {
    _log.debug("selectedAppProfileChange selected $selectedItem");
    _resetFormValues();
    if (selectedItem != null) {
      _oldAppProfile = selectedItem;
      selectedAppProfile = selectedItem.clone();

      formHeaderValue = interpret(SwgConfigLocaleInfo.EDIT_APP_PROFILE);
      operation = ChangeAction.EDIT;
      rightPanelOpened = true;

      if (selectedAppProfile.profileContent != null && selectedAppProfile.profileContent.isNotEmpty) {
        List copyList = new List.from(selectedAppProfile.profileContent
            .map((appProfileData) => new AppControlItemAction.fromAppProfileData(appProfileData)));
        controlItemGridProvider = copyList;
      } else {
        controlItemGridProvider = [];
      }
    }
  }

  onSelectItemIndex(int selectedItemIndex) {
    selectedIndex = selectedItemIndex;
  }

  onSubmitAppProfileForm(FormEvent event) {
    FormComponent form = event.form;

    int inv = 0;
    form.fields.forEach((field) {
      if (!field.validate()) inv++;
    });

    if (inv > 0) {
      form.setFormState(error: true, message: interpret(SwgConfigLocaleInfo.INVALID_FORM));
      return;
    }

    if (selectedAppProfile.profileContent == null || selectedAppProfile.profileContent.isEmpty) {
      form.setFormState(error: true, message: interpret(SwgConfigLocaleInfo.APP_PROFILE_CONTROL_ITEM_REQUIRED));
      return;
    }

    if (selectedAppProfile == _oldAppProfile) {
      _log.debug("+++++ NOTHING CHANGED +++++");
      _resetFormValues();
      _resetDetailViewValues();
      return;
    }

    List copy = new List.from(appProfiles);
    if (operation == ChangeAction.ADD) {
      copy.add(selectedAppProfile);
      copy.sort();
    } else if (operation == ChangeAction.EDIT) {
      copy.remove(_oldAppProfile);
      copy.add(selectedAppProfile);
    }

    appProfiles.clear();
    appProfiles = copy;
    appProfiles.sort();

    _resetFormValues();
    _resetDetailViewValues();
  }

  onResetAppProfileForm(FormEvent event) {
    _log.debug("onFormReset");
    _resetFormValues();
    _resetDetailViewValues();
  }

  onAddControlItem(DatagridAddEvent addEvent) {
    dialogLabel = interpret(SwgConfigLocaleInfo.ADD_CONTROL_ITEM);
    selectedControlItem = new AppControlItemAction();
    controlItemAction = ChangeAction.ADD;
    controlItemAddEditDialogVisible = true;
  }

  onEditControlItem(DatagridEditEvent editEvent) {
    _oldControlItem = editEvent.selectedItem;
    selectedControlItem = _oldControlItem.clone();

    dialogLabel = interpret(SwgConfigLocaleInfo.EDIT_CONTROL_ITEM);
    controlItemAction = ChangeAction.EDIT;
    controlItemAddEditDialogVisible = true;

    selectedControlItem.action != null ? actionSelectionModel.select(selectedControlItem.action) : '';
    selectedControlItem.applicationName != null
        ? applicationSelectionModel.select(selectedControlItem.applicationName)
        : '';
  }

  onDeleteControlItem(DatagridDeleteEvent deleteEvent) {
    if (controlItemGrid.checkedItems != null && controlItemGrid.checkedItems.isNotEmpty) {
      List<AppControlItemAction> copyList = new List.from(controlItemGridProvider);
      controlItemGrid.checkedItems.forEach((toDelete) {
        copyList.removeWhere((controlItem) =>
            controlItem.applicationName == toDelete.applicationName &&
            controlItem.itemName == toDelete.itemName &&
            controlItem.action == toDelete.action);
      });
      controlItemGridProvider = copyList;
      List applicationContents = new List.from(
          copyList.map((appControlItemAction) => new AppProfileData.fromAppControlItemAction(appControlItemAction)));
      selectedAppProfile.profileContent = applicationContents;
    }
  }

  onSubmitControlItemAddEdit() {
    if (!controlItemAddEditForm.validate()) {
      controlItemFormValid = false;
      controlItemFormValidationMessage = interpret(SwgConfigLocaleInfo.INVALID_FORM);
      return;
    }

    selectedControlItem.action = selectedAction;
    selectedControlItem.applicationName = applicationSelectLabel;
    selectedControlItem.itemName = itemSelectLabel;

    if (controlItemAction == ChangeAction.ADD) {
      List copy = new List.from(controlItemGridProvider);
      copy.add(selectedControlItem.clone());
      copy.sort((a, b) => a.applicationName.toLowerCase().compareTo(b.applicationName.toLowerCase()));
      controlItemGridProvider.clear();
      controlItemGridProvider = copy;
    } else if (controlItemAction == ChangeAction.EDIT) {
      List copy = new List.from(controlItemGridProvider);
      copy.removeWhere((controlItem) =>
          controlItem.applicationName == _oldControlItem.applicationName &&
          controlItem.itemName == _oldControlItem.itemName &&
          controlItem.action == _oldControlItem.action);
      copy.add(selectedControlItem.clone());
      copy.sort((a, b) => a.applicationName.toLowerCase().compareTo(b.applicationName.toLowerCase()));
      controlItemGridProvider.clear();
      controlItemGridProvider = copy;
    }
    List copyList = new List.from(controlItemGridProvider
        .map((appControlItemAction) => new AppProfileData.fromAppControlItemAction(appControlItemAction)));
    selectedAppProfile.profileContent = copyList;
    onCancelControlItemAddEdit();
  }

  onCancelControlItemAddEdit() {
    controlItemAddEditForm.reset();
    controlItemFormValid = true;
    controlItemFormValidationMessage = "";
    controlItemAddEditDialogVisible = false;

    actionSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  }

  ///
  /// Template Members: Functions
  ///

  String validateAppProfileName(String value) {
    if (appProfiles
        .any((summary) => summary.name.toLowerCase() == value.toLowerCase() && selectedAppProfile.id != summary.id)) {
      return interpret(SwgConfigLocaleInfo.APP_PROFILE_NAME_EXISTS);
    }
    return "";
  }

  ///
  /// Lifecycle Hooks
  ///
  @override
  onNgInit() async {
    _log.debug("+++++ onNgInit +++++");
    _loadData();
    _loadActionItems();
    _getAvailableAppControlItems();
    applicationSelectionModel.selectionChanges.listen(_updateApplicationItems);
  }

  ///
  /// Private Members
  ///

  AppControlProfile _oldAppProfile;
  AppControlItemAction _oldControlItem;

  _resetFormValues() {
    selectedAppProfile = null;
    controlItemGridProvider = null;
    selectedControlItem = null;
    _oldAppProfile = null;
    _oldControlItem = null;
  }

  _resetDetailViewValues() {
    rightPanelOpened = false;
    appProfilesGrid.selectedItem = null;
    appProfilesGrid.selectedIndex = null;
  }

  _loadData() async {
    _log.debug("_loadData");
    await _getAppProfilesForCustomer();
  }

  _getAppProfilesForCustomer() async {
    if (CustomerModelLocator.customer != null) {
      List copyList = await _buildingBlockService.getAvailableAppControlProfiles(CustomerModelLocator.customer.id);
      copyList.sort();
      appProfiles = copyList ?? [];
    }
  }

  _loadActionItems() async {
    if (actionsDataProvider == null || actionsDataProvider.isEmpty) {
      _buildingBlockService.getAllowedAppControlItemActions().then((value) {
        actionsDataProvider = value;
      }).catchError((error) {
        actionsDataProvider = [];
      });
    }
  }

  _getAvailableAppControlItems() async {
    if (_appControlItems.isEmpty) {
      List availableAppControlItems = await _buildingBlockService.getAvailableAppControlItems();
      if (availableAppControlItems != null) {
        for (AppControlItem item in availableAppControlItems) {
          if (!_appControlItems.containsKey(item.applicationName)) {
            _appControlItems[item.applicationName] = [item.itemName];
          } else if (!_appControlItems[item.applicationName].contains(item.itemName)) {
            _appControlItems[item.applicationName].add(item.itemName);
          }
        }
      }
      applicationsDataProvider = _appControlItems.keys.toList();
    }
  }

  _updateApplicationItems(List<SelectionChangeRecord> values) {
    itemSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    if (applicationSelectLabel.isNotEmpty) {
      applicationItems = _appControlItems[applicationSelectLabel];
      itemDropdownDisabled = false;
      if (selectedControlItem != null && selectedControlItem.itemName != null) {
        itemSelectionModel.select(selectedControlItem.itemName);
      }
    } else {
      itemDropdownDisabled = true;
    }
  }

  // TODO: handle error
  _handleError(e) {
    String msg = (e != null && e.message != null) ? e.message : "";
    _log.error(Error.safeToString(msg), e);
    notifyError(msg);
  }
}
