library swg.config.ui.angular.component.customer_selector;


import 'dart:html';
import 'package:angular/angular.dart';
import 'package:swg_config_ui_angular/src/common/locale.dart';
import 'package:swg_ui_common/src/model/customer.dart';
import 'package:swg_ui_common/src/service/tenant_service.dart';
import 'package:swg_ui_common/src/util/customer_model_locator.dart';
import 'package:tw_portal/portal.dart';
import 'package:tw_ui_angular/tw_ui_angular.dart';
import 'package:tw_ui_angular/src/components/autocomplete/autocomplete.dart';
import 'package:angular_components/angular_components.dart';

@Component(selector: 'twa-swg-customer-selector',
    templateUrl: 'customer_selector.html',
    styleUrls: const ['customer_selector.css'],
    pipes: const [InterpretPipe],
    exports: const [SwgConfigLocaleInfo],
    directives: const [NgFor, NgClass, twaDirectives],
    providers: const [const Provider(ComponentService, useFactory: getComponentService),
               const Provider(ComponentBase, useExisting: CustomerSelectorComponent),
               tenantSvcProviders]
)
class CustomerSelectorComponent extends PortalComponent {

  final Logger _log = new Logger("CustomerSelectorComponent");

  static final int _minCharSearch = 2;

  //
  // Services & Constructor
  //
  final TenantService _tenantService;

  CustomerSelectorComponent(ComponentService cmpSvc, this._tenantService) : super(cmpSvc);

  @override
  final List<String> localeRoots = [ SwgConfigLocaleInfo.PORTAL_KEY, SwgConfigLocaleInfo.SWG_CONFIG_COMMON_KEY];

  //
  // Template Members
  //

  String searchTerm;
  String selectButtonState = "";
  String selectButtonMessage = "";
  String searchButtonState = "";
  String searchButtonMessage = "";
  bool blockPage = false;
  List customers = [];
  Customer selectedCustomer;

  String validateSearchTerm(String searchString) {
    if (searchString == null || searchString.isEmpty || searchString.length <= _minCharSearch) {
      return interpret(SwgConfigLocaleInfo.CUSTOMER_INPUT_ERROR_MESSAGE, [_minCharSearch]);
    }
    return "";
  }

  searchCustomers(Event e) async {
    _log.debug("searchCustomers $e");
    searchButtonState = ButtonComponent.NORMAL_STATE;
    searchButtonMessage = "";
    if (searchTerm == null || searchTerm.trim().isEmpty || searchTerm.length <= _minCharSearch) {
      searchButtonState = ButtonComponent.ERROR_STATE;
      searchButtonMessage = interpret(SwgConfigLocaleInfo.CUSTOMER_INPUT_ERROR_MESSAGE, [_minCharSearch]);
      return;
    }
    customers = [];
    try {
      blockPage = true;
      customers = await _tenantService.getProvisionedCustomers(searchTerm);
      customers ?? [];
    } catch (e) {
      _handleError(e);
      customers = [];
    } finally {
      blockPage = false;
    }

  }

  applySelectedCustomer(Event e) async {
    _log.debug("selectCustomer $e selectedCustomer $selectedCustomer");
    selectButtonState = ButtonComponent.NORMAL_STATE;
    selectButtonMessage = "";
    if (selectedCustomer == null) {
      selectButtonState = ButtonComponent.ERROR_STATE;
      selectButtonMessage = interpret(SwgConfigLocaleInfo.SELECT_CUSTOMER_ERROR_MESSAGE);
      return;
    }
    _log.debug("navigateToMenu selectedCustomer $selectedCustomer");
    CustomerModelLocator.customer = selectedCustomer;
    navigateToMenu("swg-customer-identities");
    /*navigateInStack("swg-settings",
        params: {'customer': selectedCustomer});*/
  }

  selectedCustomerChange(dynamic item) {
    _log.debug("selectedCustomer $selectedCustomer item $item");
    selectedCustomer = item;
    _log.debug("selectedCustomer $selectedCustomer");
  }

  reset(Event e) {
    _log.debug("reset $e");
    searchTerm = "";
    customers = [];
  }

  //
  // Lifecycle Hooks
  //

  @override
  onNgInit() async {}



  //
  // Private Members
  //

  _handleError(e) {
    String msg = (e != null && e.message != null) ? e.message : "";
    _log.error(Error.safeToString(msg), e);
    notifyError(msg);
  }


}