import 'dart:html';
import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:swg_config_ui_angular/src/common/locale.dart';
import 'package:swg_ui_common/src/model/change_action.dart';
import 'package:swg_ui_common/src/model/customer.dart';
import 'package:swg_ui_common/src/model/identifier.dart';
import 'package:swg_ui_common/src/model/identifier_type.dart';
import 'package:swg_ui_common/src/model/identity.dart';
import 'package:swg_ui_common/src/model/msc_definition.dart';
import 'package:swg_ui_common/src/model/named_policy_summary.dart';
import 'package:swg_ui_common/src/model/user_identity.dart';
import 'package:swg_ui_common/src/service/identifier_cache.dart';
import 'package:swg_ui_common/src/service/identity_service.dart';
import 'package:swg_ui_common/src/service/tenant_service.dart';
import 'package:swg_ui_common/src/util/customer_model_locator.dart';
import 'package:tw_portal/portal.dart';
import 'package:tw_ui_angular/tw_ui_angular.dart';



class BaseIdentitiesComponent {

  final Logger _log = new Logger("BaseIdentitiesComponent");

  final int nameLength = 100;

  List get booleanValues => [null, true, false];
  List<Identifier> namedPolicies = [];
  List<String> timeZonesDataProvider = [];
  List identities = [];
  List identityRules = [];
  List<RelativePosition> get alphaPositions => const [
    RelativePosition.AdjacentBottomLeft,
    RelativePosition.AdjacentBottomRight,
    RelativePosition.AdjacentTopLeft,
    RelativePosition.AdjacentTopRight
  ];

  bool rightPanelOpened = false;
  bool nothingSelected = true;
  bool waiting = false;
  bool isEditing = false;
  bool identityUpdatedOrDeleted = false;
  bool mscDefinitionUpdatedOrDeleted = false;

  String formHeaderValue = "";
  String identifier = "";
  String fieldWidth = "150";
  String fieldWidth2 = "100";
  String formMessage = "";
  String formState = ButtonComponent.NORMAL_STATE;
  String selectedTimeZone;
  String get policySelectLabel => policySelectionModel.selectedValues.length > 0
      ? policyItemRenderer(policySelectionModel.selectedValues.first)
      : "";
  String get safeSearchSelectLabel => safeSearchSelectionModel.selectedValues.length > 0
      ? commonItemRenderer(safeSearchSelectionModel.selectedValues.first)
      : "";
  String get preventDisablingSelectLabel => preventDisablingSelectionModel.selectedValues.length > 0
      ? commonItemRenderer(preventDisablingSelectionModel.selectedValues.first)
      : "";
  String get autoUpgradeSelectLabel => autoUpgradeSelectionModel.selectedValues.length > 0
      ? commonItemRenderer(autoUpgradeSelectionModel.selectedValues.first)
      : "";

  SelectionModel timezoneSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel<Identifier> policySelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel safeSearchSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel preventDisablingSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel autoUpgradeSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);

  SelectionOptions<Identifier> get policies => new SelectionOptions.fromList(namedPolicies ?? []);
  SelectionOptions get booleanData => new SelectionOptions.fromList(booleanValues);

  ItemRenderer<Identifier> policyItemRenderer = new CachingItemRenderer<Identifier>((identifier) => "${identifier.name}");
  ItemRenderer commonItemRenderer = new CachingItemRenderer((val) => (val == null) ? "" : val ? "Yes": "No");

  Identifier selectedPolicyIdentifier;
  RegExp badChars = null;
  Identity selected;
  Identity newIdentity;
  int selectedIndex;
  ChangeAction operation;


  Map assignedPolicyValueFunction(Identifier dataValue, String dataField, int index, dynamic item) {
    _log.debug("dataValue $dataValue dataField, $dataField index $index item $item");
    Map retValue = new Map();
    retValue['value'] = dataValue != null ? dataValue.name : "";
    retValue['tooltip'] = dataValue != null ? dataValue.name : "";
    return retValue;
  }

  Map ruleCountValueFunction(int count, String dataField, int index, dynamic item) {
    Map retValue = new Map();
    List rules = (item as Identity).identityRules;
    retValue['value'] = rules != null ? rules.length > 0 ? rules.length : "" : "";
    retValue['tooltip'] = rules != null ? rules.length > 0 ? rules.length : "" : "";
    return retValue;
  }

  String validateIdentityName(String value) {
    String msg = _isValidName(value);
    if (msg.isNotEmpty) {
      return msg;
    }
    // not required any more - component handles empty
    /*if (selected == null || selected.identifier == null || selected.identifier.isEmpty) {
      return "Identity must have a name";
    }*/

    if ((ChangeAction.EDIT == operation && selected != null
        && selected.identifier != newIdentity.identifier) || operation == ChangeAction.ADD) {
      if (_checkIdentityExists(newIdentity.identifier)) {
        msg = "Identity already exists";
        return msg;
      }
    }
    return "";
  }

  String validateNameExists(String fieldName, String value, Function localeFunction) {
    String msg = "";
    if ((!isEditing && selected != null && selected.identifier != newIdentity.identifier) || operation == ChangeAction.ADD) {
      if (identities.any((Identity vo) => vo.identifier == value)) {
        msg = localeFunction(SwgConfigLocaleInfo.NAME_EXISTS, [fieldName]);
      }
    }
    return msg;
  }

  onTimezoneSelectionChange(AutoCompleteEvent e) {
    if (e != null && e.selectedItems != null && e.selectedItems.isNotEmpty) {
      selectedTimeZone = e.selectedItems.first;
    }
  }

  bool _checkIdentityExists(String identifier) {
    for (Identity identity in identities) {
      if (identity != null && identity.identifier != null && identifier != null
          && identity.identifier.toLowerCase() == identifier.toLowerCase()) {
        return true;
      }
    }
    return false;
  }

  String _isValidName(String value) {
    if (badChars == null) {
      return "";
    }
    List<Match> matches = badChars.allMatches(value);
    if (matches.isNotEmpty) {
      Set<String> matchSet = new Set.from(matches.map((match) => match.group(0)));
      return "Identity name cannot contain the following special character"
          "${matchSet.length == 1 ? ": ${matchSet.first}"
          : "s: ${_setToString(matchSet)}"}";

    }
    return "";
  }

  String _setToString(Set set) {
    StringBuffer sb = new StringBuffer();
    for (var element in set) {
      sb.write(element);
    }
    return sb.toString();
  }

  getDropDownSelectedItem(SelectionModel selModel) {
    return selModel == null ? null : selModel.selectedValues.length > 0 ? selModel.selectedValues.first : null;
  }
}