library swg.config.ui.angular.component.group_identities;

import 'dart:html';
import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:swg_config_ui_angular/src/common/locale.dart';
import 'package:swg_config_ui_angular/src/ui/component/identities/base_identities.dart';
import 'package:swg_config_ui_angular/src/ui/component/rule/rule_common.dart';
import 'package:swg_ui_common/src/model/change_action.dart';
import 'package:swg_ui_common/src/model/customer.dart';
import 'package:swg_ui_common/src/model/group_identity.dart';
import 'package:swg_ui_common/src/model/identifier_type.dart';
import 'package:swg_ui_common/src/model/identity.dart';
import 'package:swg_ui_common/src/model/msc_definition.dart';
import 'package:swg_ui_common/src/service/identifier_cache.dart';
import 'package:swg_ui_common/src/service/identity_service.dart';
import 'package:swg_ui_common/src/util/customer_model_locator.dart';
import 'package:tw_portal/portal.dart';
import 'package:tw_ui_angular/tw_ui_angular.dart';

@Component(selector: 'twa-swg-group-identities',
    templateUrl: 'group_identities.html',
    styleUrls: const ['group_identities.css'],
    pipes: const [InterpretPipe],
    exports: const [SwgConfigLocaleInfo, ChangeAction],
    directives: const [twaDirectives, RulesComponent],
    providers: const [const Provider(ComponentService, useFactory: getComponentService),
    const Provider(ComponentBase, useExisting: GroupIdentitiesComponent)]
)
class GroupIdentitiesComponent extends PortalComponent with BaseIdentitiesComponent {

  final Logger _log = new Logger("GroupIdentitiesComponent");

  static const int PRIORITY_MIN = 1;
  static const int PRIORITY_MAX = 9999;

  GroupIdentitiesComponent(ComponentService cmpSvc, this._identityService, this._identifierCache) : super(cmpSvc);

  final RpcIdentityService _identityService;
  final IdentifierCache _identifierCache;

  @override
  final List<String> localeRoots = [ SwgConfigLocaleInfo.PORTAL_KEY, SwgConfigLocaleInfo.SWG_CONFIG_KEY, SwgConfigLocaleInfo.SWG_CONFIG_COMMON_KEY];

  bool waiting = false;
  int priority;
  RegExp badChars = new RegExp(r'[,±~#$%^&(){}[\]"?><*+=|;]');

  @override
  onNgInit() async {
    _loadData();
  }

  @override
  onLocaleLoaded() {

  }

  onAdd(DatagridAddEvent addEvent) {
    _log.debug("onAdd addEvent $addEvent");
    _log.debug("onAdd addEvent $addEvent");
    formHeaderValue = interpret(SwgConfigLocaleInfo.ADD_GROUP);
    _resetFormValues();
    newIdentity = new GroupIdentity();
    operation = ChangeAction.ADD;
    isEditing = false;
    rightPanelOpened = true;
    nothingSelected = false;
  }

  onDelete(DatagridDeleteEvent deleteEvent) {
    _log.debug("onAdd DatagridDeleteEvent $deleteEvent");
    identities.remove(deleteEvent.selectedItem);
    _resetFormValues();
    _resetDetailViewValues();
  }

  onRefresh(DatagridRefreshEvent refreshEvent) {
    _resetFormValues();
    rightPanelOpened = false;
    nothingSelected = true;
    _loadData();
  }

  onSelectItem(GroupIdentity selectedItem) {
    _log.debug("selectedUserIdentityChange selected $selected");
    _resetFormValues();
    if (selectedItem != null) {
      selected = selectedItem;
      newIdentity = selected.clone();
      formHeaderValue = interpret(SwgConfigLocaleInfo.EDIT_GROUP);
      identifier = newIdentity.identifier;
      newIdentity.policyIdentifier != null ? policySelectionModel.select(newIdentity.policyIdentifier) : '';
      newIdentity.timezone != null ? timezoneSelectionModel.select(newIdentity.timezone) : '';
      newIdentity.safeSearch != null ? safeSearchSelectionModel.select(newIdentity.safeSearch) : '';
      if ((newIdentity as GroupIdentity).mscDefinition != null) {
        (newIdentity as GroupIdentity).mscDefinition.preventDisabling != null ?
        preventDisablingSelectionModel.select((newIdentity as GroupIdentity).mscDefinition.preventDisabling) : '';
        (newIdentity as GroupIdentity).mscDefinition.allowUpgrade != null ?
        autoUpgradeSelectionModel.select((newIdentity as GroupIdentity).mscDefinition.allowUpgrade) : '';
      }
      identityRules = new List.from(selected.identityRules ?? []);
      priority = selectedItem.priority;
      operation = ChangeAction.EDIT;
      isEditing = true;
      rightPanelOpened = true;
      nothingSelected = false;
      operation = ChangeAction.EDIT;
      isEditing = true;
    }
  }

  onSelectItemIndex(int selectedItemIndex) {
    selectedIndex = selectedItemIndex;
  }

  onFormSubmit() async {
    _log.debug("onFormSubmit");
    formMessage = "";
    formState = ButtonComponent.NORMAL_STATE;
    _log.debug("onFormSubmit");
    newIdentity.identifier = identifier;
    Identity identity = await _identityService.getIdentityWithSameName(CustomerModelLocator.customer.id, newIdentity);
    //bool isIdentityNameInUse = _isIdentityWithSameNameExists(identity);
    if (identity != null) {
      formMessage = interpret(SwgConfigLocaleInfo.IDENTITY_WITH_SAME_NAME_AND_DIFFERENT_TYPE_EXISTS);
      formState = ButtonComponent.ERROR_STATE;
      return;
    }
    newIdentity.policyIdentifier = getDropDownSelectedItem(policySelectionModel);
    newIdentity.timezone = getDropDownSelectedItem(timezoneSelectionModel);
    newIdentity.safeSearch = getDropDownSelectedItem(safeSearchSelectionModel);
    newIdentity.identityRules = new List.from(identityRules ?? []);
    bool preventDisabling = getDropDownSelectedItem(preventDisablingSelectionModel);
    bool autoUpgrade = getDropDownSelectedItem(autoUpgradeSelectionModel);
    (newIdentity as GroupIdentity).priority = priority;
    if (operation == ChangeAction.ADD) {
      if (preventDisabling != null || autoUpgrade != null) {
        MscDefinition mscDefinition =  new MscDefinition();
        mscDefinition.preventDisabling = preventDisabling;
        mscDefinition.allowUpgrade = autoUpgrade;
        (newIdentity as GroupIdentity).mscDefinition = mscDefinition;
        (newIdentity as GroupIdentity).mscDefinitionUpdated = true;
      }
      List identitiesCopy = new List.from(identities);
      identitiesCopy.add(newIdentity);
      identities.clear();
      identities = identitiesCopy;
    } else {
      if ((selected as GroupIdentity).mscDefinition != (newIdentity as GroupIdentity).mscDefinition) {
        (newIdentity as GroupIdentity).mscDefinition.preventDisabling = preventDisabling;
        (newIdentity as GroupIdentity).mscDefinition.allowUpgrade = autoUpgrade;
        (newIdentity as GroupIdentity).mscDefinitionUpdated = true;
      }
      List identitiesCopy = new List.from(identities);
      identitiesCopy.remove(selected);
      identitiesCopy.add(newIdentity);
      identities.clear();
      identities = identitiesCopy;
    }

    //ChangeAction action = isEditing ? ChangeAction.EDIT : ChangeAction.ADD;
    if (selected != null && selected.policyIdentifier != newIdentity.policyIdentifier) {
      newIdentity.parentUpdated = true;
    }

    //handleMscSettings();

    if ((selected == null && newIdentity.policyIdentifier != null)
        || (selected != null && selected.policyIdentifier != newIdentity.policyIdentifier)) {
      identityUpdatedOrDeleted = true;
    }
    _log.debug("selectedIndex $selectedIndex newIdentity $newIdentity");

    // this is not working
    //userIdentitiesGrid.updateRow(selectedIndex, newIdentity);
    _resetFormValues();
    _resetDetailViewValues();
  }

  onFormReset() {
    _log.debug("onFormReset");
    _resetFormValues();
    _resetDetailViewValues();
  }

  String validatePriority(int priority) {
    String message = "";
    if (priority < PRIORITY_MIN || priority > PRIORITY_MAX) {
      message = "The priority must be greater than $PRIORITY_MIN but not greater than $PRIORITY_MAX";
    } else if (identities != null && identities.isNotEmpty && priority != null
        && (selected == null || (selected != null && (selected as GroupIdentity).priority != (newIdentity as GroupIdentity).priority))) {
      var value = identities.firstWhere((identity) => identity.priority == priority  , orElse: () => null);
      if (value != null) {
        message = "This priority is used by another group";
      }
    }
    return message;
  }

  _resetFormValues() {
    identifier = "";
    priority = null;
    identityRules = [];
    policySelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    safeSearchSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    timezoneSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    preventDisablingSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    autoUpgradeSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  }

  _resetDetailViewValues() {
    rightPanelOpened = false;
    nothingSelected = true;
  }


  _loadData() async {
    _log.debug("_loadData");
    await _loadIdentities();
    _loadTimeZones();
    _loadPolicies();
  }

  _loadPolicies() async {
    List policies = new List.from(await _identifierCache.getItemsOfType(IdentifierType.NAMED_POLICY));
    policies.sort();
    namedPolicies = policies;
  }

  _loadIdentities() async {
    try {
      waiting = true;
      List list = await _identityService.getGroupIdentitiesForCustomer(CustomerModelLocator?.customer.id);
      identities = list ?? [];
      _log.debug("_loadData groupIdentities $identities");
    } catch (e) {
      _handleError(e);
      identities = [];
    } finally {
      waiting = false;
    }
  }

  _loadTimeZones() async {
    timeZonesDataProvider = await _identifierCache.getTimeZones();
  }

  _handleError(e) {
    String msg = (e != null && e.message != null) ? e.message : "";
    _log.error(Error.safeToString(msg), e);
    notifyError(msg);
  }
}