library swg.config.ui.angular.component.user_identities;


import 'dart:html';
import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:swg_config_ui_angular/src/common/locale.dart';
import 'package:swg_config_ui_angular/src/ui/component/identities/base_identities.dart';
import 'package:swg_config_ui_angular/src/ui/component/rule/rule_common.dart';
import 'package:swg_ui_common/src/model/change_action.dart';
import 'package:swg_ui_common/src/model/identifier.dart';
import 'package:swg_ui_common/src/model/identifier_type.dart';
import 'package:swg_ui_common/src/model/identity.dart';
import 'package:swg_ui_common/src/model/msc_definition.dart';
import 'package:swg_ui_common/src/model/user_identity.dart';
import 'package:swg_ui_common/src/service/identifier_cache.dart';
import 'package:swg_ui_common/src/service/identity_service.dart';
import 'package:swg_ui_common/src/util/customer_model_locator.dart';
import 'package:tw_portal/portal.dart';
import 'package:tw_ui_angular/tw_ui_angular.dart';


@Component(selector: 'twa-swg-user-identities',
    templateUrl: 'user_identities.html',
    styleUrls: const ['user_identities.css'],
    pipes: const [InterpretPipe],
    exports: const [SwgConfigLocaleInfo, ChangeAction],
    directives: const [twaDirectives, RulesComponent],
    providers: const [const Provider(ComponentService, useFactory: getComponentService),
    const Provider(ComponentBase, useExisting: UserIdentitiesComponent)]
)
class UserIdentitiesComponent extends PortalComponent with BaseIdentitiesComponent {

  final Logger _log = new Logger("UserIdentitiesComponent");

  //
  // Services & Constructor
  //

  UserIdentitiesComponent(ComponentService cmpSvc, this._identityService, this._identifierCache) : super(cmpSvc);

  final RpcIdentityService _identityService;
  final IdentifierCache _identifierCache;

  @override
  final List<String> localeRoots = [ SwgConfigLocaleInfo.PORTAL_KEY, SwgConfigLocaleInfo.SWG_CONFIG_KEY, SwgConfigLocaleInfo.SWG_CONFIG_COMMON_KEY];

  @override
  onNgInit() async {
    _loadData();
  }

  @override
  onLocaleLoaded() {

  }

  RegExp badChars = new RegExp(r'[,:±~#$%^&*(){}[\]]');


  @ViewChild('userIdentitiesGrid')
  DatagridComponent userIdentitiesGrid;

  onAdd(DatagridAddEvent addEvent) {
    _log.debug("onAdd addEvent $addEvent");
    formHeaderValue = interpret(SwgConfigLocaleInfo.ADD_USER);
    _resetFormValues();
    newIdentity = new UserIdentity();
    operation = ChangeAction.ADD;
    isEditing = false;
    rightPanelOpened = true;
    nothingSelected = false;
  }

  onDelete(DatagridDeleteEvent deleteEvent) {
    _log.debug("onAdd DatagridDeleteEvent $deleteEvent");
    var deleteItem = deleteEvent.selectedItem;
    identities.remove(deleteItem);
    _resetFormValues();
    _resetDetailViewValues();
  }

  onRefresh(DatagridRefreshEvent refreshEvent) {
    _log.debug("onRefresh refreshEvent $refreshEvent");
    _resetFormValues();
    rightPanelOpened = false;
    nothingSelected = true;
    _loadData();
  }

  onSelectItem(UserIdentity selectedItem) {
    _log.debug("selectedUserIdentityChange selected $selected");
    _resetFormValues();
    if (selectedItem != null) {
      selected = selectedItem;
      newIdentity = selected.clone();
      formHeaderValue = interpret(SwgConfigLocaleInfo.EDIT_USER);
      identifier = newIdentity.identifier;
      newIdentity.policyIdentifier != null ? policySelectionModel.select(newIdentity.policyIdentifier) : '';
      newIdentity.timezone != null ? timezoneSelectionModel.select(newIdentity.timezone) : '';
      newIdentity.safeSearch != null ? safeSearchSelectionModel.select(newIdentity.safeSearch) : '';
      if ((newIdentity as UserIdentity).mscDefinition != null) {
        (newIdentity as UserIdentity).mscDefinition.preventDisabling != null ?
        preventDisablingSelectionModel.select((newIdentity as UserIdentity).mscDefinition.preventDisabling) : '';
        (newIdentity as UserIdentity).mscDefinition.allowUpgrade != null ?
        autoUpgradeSelectionModel.select((newIdentity as UserIdentity).mscDefinition.allowUpgrade) : '';
      }
      identityRules = new List.from(selected.identityRules ?? []);
      operation = ChangeAction.EDIT;
      isEditing = true;
      rightPanelOpened = true;
      nothingSelected = false;
      operation = ChangeAction.EDIT;
      isEditing = true;
    }
  }

  onSelectItemIndex(int selectedItemIndex) {
    selectedIndex = selectedItemIndex;
  }


  String validateIdentityName(String value) {
    if (value.contains(" ")) {
      return "User ID cannot contain spaces";
    }
    return super.validateIdentityName(value);
  }

  onFormSubmit() async {
    formMessage = "";
    formState = ButtonComponent.NORMAL_STATE;
    _log.debug("onFormSubmit");
    newIdentity.identifier = identifier;
    Identity identity = await _identityService.getIdentityWithSameName(CustomerModelLocator.customer.id, newIdentity);
    //bool isIdentityNameInUse = _isIdentityWithSameNameExists(identity);
    if (identity != null) {
      formMessage = interpret(SwgConfigLocaleInfo.IDENTITY_WITH_SAME_NAME_AND_DIFFERENT_TYPE_EXISTS);
      formState = ButtonComponent.ERROR_STATE;
      return;
    }
    newIdentity.policyIdentifier = getDropDownSelectedItem(policySelectionModel);
    newIdentity.timezone = getDropDownSelectedItem(timezoneSelectionModel);
    newIdentity.safeSearch = getDropDownSelectedItem(safeSearchSelectionModel);
    newIdentity.identityRules = new List.from(identityRules ?? []);
    bool preventDisabling = getDropDownSelectedItem(preventDisablingSelectionModel);
    bool autoUpgrade = getDropDownSelectedItem(autoUpgradeSelectionModel);

    if (operation == ChangeAction.ADD) {
      if (preventDisabling != null || autoUpgrade != null) {
        MscDefinition mscDefinition =  new MscDefinition();
        mscDefinition.preventDisabling = preventDisabling;
        mscDefinition.allowUpgrade = autoUpgrade;
        (newIdentity as UserIdentity).mscDefinition = mscDefinition;
        (newIdentity as UserIdentity).mscDefinitionUpdated = true;
      }
      List identitiesCopy = new List.from(identities);
      identitiesCopy.add(newIdentity);
      identities.clear();
      identities = identitiesCopy;
    } else {
      if ((selected as UserIdentity).mscDefinition != (newIdentity as UserIdentity).mscDefinition) {
        (newIdentity as UserIdentity).mscDefinition.preventDisabling = preventDisabling;
        (newIdentity as UserIdentity).mscDefinition.allowUpgrade = autoUpgrade;
        (newIdentity as UserIdentity).mscDefinitionUpdated = true;
      }
      List identitiesCopy = new List.from(identities);
      identitiesCopy.remove(selected);
      identitiesCopy.add(newIdentity);
      identities.clear();
      identities = identitiesCopy;
    }

    //ChangeAction action = isEditing ? ChangeAction.EDIT : ChangeAction.ADD;
    if (selected != null && selected.policyIdentifier != newIdentity.policyIdentifier) {
      newIdentity.parentUpdated = true;
    }

    //handleMscSettings();

    if ((selected == null && newIdentity.policyIdentifier != null)
        || (selected != null && selected.policyIdentifier != newIdentity.policyIdentifier)) {
      identityUpdatedOrDeleted = true;
    }
    _log.debug("selectedIndex $selectedIndex newIdentity $newIdentity");

    // this is not working
    //userIdentitiesGrid.updateRow(selectedIndex, newIdentity);
    _resetFormValues();
    _resetDetailViewValues();
  }

  void handleMscSettings() {
    if (selected != null && selected is UserIdentity && newIdentity is UserIdentity) {
      if ((selected as UserIdentity).mscDefinition != null && (newIdentity as UserIdentity).mscDefinition != null &&
          (selected as UserIdentity).mscDefinition != (newIdentity as UserIdentity).mscDefinition) {
        (newIdentity as UserIdentity).mscDefinitionUpdated = true;
        mscDefinitionUpdatedOrDeleted = true;
      } else if ((selected as UserIdentity).mscDefinition == null) {
        if ((newIdentity as UserIdentity).mscDefinition != null
            && ((newIdentity as UserIdentity).mscDefinition.preventDisabling != null
                || (newIdentity as UserIdentity).mscDefinition.allowUpgrade != null)) {
          (newIdentity as UserIdentity).mscDefinitionUpdated = true;
          mscDefinitionUpdatedOrDeleted = true;
        } else {
          (newIdentity as UserIdentity).mscDefinition = null;
        }
      }
    } else if (selected == null && newIdentity is UserIdentity) {
      if ((newIdentity as UserIdentity).mscDefinition != null && ((newIdentity as UserIdentity).mscDefinition.preventDisabling != null
          || (newIdentity as UserIdentity).mscDefinition.allowUpgrade != null)) {
        (newIdentity as UserIdentity).mscDefinitionUpdated = true;
        mscDefinitionUpdatedOrDeleted = true;
      }
    }
  }

  onFormReset() {
    _resetFormValues();
    _log.debug("onFormReset");
    _resetDetailViewValues();
  }

  _resetFormValues() {
    identifier = "";
    identityRules = [];
    policySelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    safeSearchSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    timezoneSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    preventDisablingSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
    autoUpgradeSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  }

  _resetDetailViewValues() {
    rightPanelOpened = false;
    nothingSelected = true;
  }

  _loadData() async {
    _log.debug("_loadData");
    await _loadIdentities();
    _loadTimeZones();
    _loadPolicies();
  }

  _loadPolicies() async {
    List policies = new List.from(await _identifierCache.getItemsOfType(IdentifierType.NAMED_POLICY));
    policies.sort();
    namedPolicies = policies;
  }

  _loadIdentities() async {
    try {
      waiting = true;
      List list = await _identityService.getUserIdentitiesForCustomer(CustomerModelLocator?.customer.id);
      identities = list ?? [];
      _log.debug("_loadData userIdentities $identities");
    } catch (e) {
      _handleError(e);
      identities = [];
    } finally {
      waiting = false;
    }
  }

  _handleError(e) {
    String msg = (e != null && e.message != null) ? e.message : "";
    _log.error(Error.safeToString(msg), e);
    notifyError(msg);
  }

  _loadTimeZones() async {
    timeZonesDataProvider = await _identifierCache.getTimeZones();
  }




}