library swg.config.ui.angular.ui.component.rule;

import 'dart:html';
import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:swg_config_ui_angular/src/common/locale.dart';
import 'package:swg_ui_common/src/model/action.dart';
import 'package:swg_ui_common/src/model/app_control_item.dart';
import 'package:swg_ui_common/src/model/app_control_profile_rule.dart';
import 'package:swg_ui_common/src/model/change.dart';
import 'package:swg_ui_common/src/model/change_action.dart';
import 'package:swg_ui_common/src/model/content_size_rule.dart';
import 'package:swg_ui_common/src/model/custom_app_control_rule.dart';
import 'package:swg_ui_common/src/model/custom_url_rule.dart';
import 'package:swg_ui_common/src/model/dlp_rule.dart';
import 'package:swg_ui_common/src/model/eum.dart';
import 'package:swg_ui_common/src/model/file_extension_group_rule.dart';
import 'package:swg_ui_common/src/model/file_extension_rule.dart';
import 'package:swg_ui_common/src/model/file_type_rule.dart';
import 'package:swg_ui_common/src/model/identifier.dart';
import 'package:swg_ui_common/src/model/identifier_type.dart';
import 'package:swg_ui_common/src/model/policy_identity_rule.dart';
import 'package:swg_ui_common/src/model/quota_rule.dart';
import 'package:swg_ui_common/src/model/rule.dart';
import 'package:swg_ui_common/src/model/rule_exception.dart';
import 'package:swg_ui_common/src/model/rule_type.dart';
import 'package:swg_ui_common/src/model/time_frame.dart';
import 'package:swg_ui_common/src/model/url_category.dart';
import 'package:swg_ui_common/src/model/url_category_rule.dart';
import 'package:swg_ui_common/src/model/url_item.dart';
import 'package:swg_ui_common/src/model/url_list_rule.dart';
import 'package:swg_ui_common/src/model/user_agent_rule.dart';
import 'package:swg_ui_common/src/service/building_block_service.dart';
import 'package:swg_ui_common/src/service/identifier_cache.dart';
import 'package:swg_ui_common/src/util/change_set.dart';
import 'package:swg_ui_common/src/util/customer_model_locator.dart';
import 'package:swg_ui_common/src/util/punycode.dart';
import 'package:swg_ui_common/src/util/rule_util.dart';
import 'package:swg_ui_common/src/util/temp_id_generator.dart';
import 'package:swg_ui_common/src/util/url_validation.dart';
import 'package:tw_portal/portal.dart';
import 'package:tw_ui_angular/tw_ui_angular.dart';


@Component(selector: 'twa-swg-rules-common',
    templateUrl: 'rule_common.html',
    styleUrls: const ['rule_common.css'],
    pipes: const [InterpretPipe],
    exports: const [SwgConfigLocaleInfo, RuleTypes, Action, ChangeAction],
    directives: const [NgIf, twaDirectives],
    providers: const [const Provider(ComponentService, useFactory: getComponentService),
    const Provider(ComponentBase, useExisting: RulesComponent),
    identifierCacheProviders, buildingBlockSvcProviders
    ]
)
class RulesComponent extends PortalComponent {

  final Logger _log = new Logger("RulesComponent");

  RulesComponent(ComponentService cmpSvc, this._identifierCache, this._buildingBlockService) : super(cmpSvc);

  final IdentifierCache _identifierCache;
  final BuildingBlockService _buildingBlockService;

  @override
  final List<String> localeRoots = [
    SwgConfigLocaleInfo.PORTAL_KEY, SwgConfigLocaleInfo.SWG_CONFIG_KEY, SwgConfigLocaleInfo.SWG_CONFIG_COMMON_KEY];

  //
  // Template Members
  //

  @Input()
  List rules;

  @Input()
  String height = "300px";

  SelectionOptions<RuleTypes> get ruleTypes => new SelectionOptions.fromList(RuleTypes.values ?? []);

  SelectionOptions<Action> get actionsOptions => new SelectionOptions.fromList(actionsProvider ?? []);

  SelectionOptions<Identifier> get timeFrameIdentifiersOptions =>
      new SelectionOptions.fromList(timeFrameIdentifiers ?? []);

  SelectionOptions<Identifier> get eumOptionalIdentifiersOptions =>
      new SelectionOptions.fromList(eumOptionalIdentifiers ?? []);

  SelectionOptions<Identifier> get applicationProfileIdentifiersOptions =>
      new SelectionOptions.fromList(appProfileIdentifiers ?? []);

  SelectionOptions<String> get applicationNamesOptions => new SelectionOptions.fromList(applicationNames ?? []);

  SelectionOptions<String> get applicationItemsOptions => new SelectionOptions.fromList(applicationItems ?? []);

  SelectionOptions<String> get dlpProfilesOptions => new SelectionOptions.fromList(dlpProfiles ?? []);

  SelectionOptions<String> get fileExtensionGroupsOptions => new SelectionOptions.fromList(fileExtensionGroups ?? []);

  SelectionOptions<String> get fileTypesOptions => new SelectionOptions.fromList(fileTypes ?? []);

  SelectionOptions<Identifier> get urlCategoriesOptions => new SelectionOptions.fromList(urlCategoryIdentifiers ?? []);

  SelectionOptions<Identifier> get urlListIdentifiersOptions => new SelectionOptions.fromList(urlListIdentifiers ?? []);

  SelectionOptions<Identifier> get quotasIdentifiersOptions => new SelectionOptions.fromList(quotasIdentifiers ?? []);

  String dialogTitle = "";
  String fieldWidth = "150";

  String get ruleTypesSelectLabel =>
      ruleTypesSelectionModel.selectedValues.length > 0
          ? ruleTypesItemRenderer(ruleTypesSelectionModel.selectedValues.first)
          : "";

  String get actionsSelectLabel =>
      actionsSelectionModel.selectedValues.length > 0
          ? actionsItemRenderer(actionsSelectionModel.selectedValues.first)
          : "";

  String get timeFrameIdentifiersSelectLabel =>
      timeFrameIdentifiersSelectionModel.selectedValues.length > 0
          ? commonIdentifiersItemRenderer(timeFrameIdentifiersSelectionModel.selectedValues.first)
          : "";

  String get eumOptionalIdentifiersSelectLabel =>
      eumOptionalIdentifiersSelectionModel.selectedValues.length > 0
          ? commonIdentifiersItemRenderer(eumOptionalIdentifiersSelectionModel.selectedValues.first)
          : "";

  String get applicationProfilesSelectLabel =>
      applicationProfilesSelectionModel.selectedValues.length > 0
          ? commonIdentifiersItemRenderer(applicationProfilesSelectionModel.selectedValues.first)
          : "";

  String get applicationNamesSelectLabel =>
      applicationNamesSelectionModel.selectedValues.length > 0
          ? applicationNamesSelectionModel.selectedValues.first
          : "";

  String get applicationItemsSelectLabel =>
      applicationItemsSelectionModel.selectedValues.length > 0
          ? applicationItemsSelectionModel.selectedValues.first
          : "";

  String get dlpProfilesSelectLabel =>
      dlpProfilesSelectionModel.selectedValues.length > 0
          ? dlpProfilesSelectionModel.selectedValues.first
          : "";

  String get fileExtensionGroupsSelectLabel =>
      fileExtensionGroupsSelectionModel.selectedValues.length > 0
          ? fileExtensionGroupsSelectionModel.selectedValues.first
          : "";

  String get fileTypesSelectLabel =>
      fileTypesSelectionModel.selectedValues.length > 0
          ? fileTypesSelectionModel.selectedValues.first
          : "";

  String get urlCategoriesSelectLabel =>
      urlCategoriesSelectionModel.selectedValues.length > 0
          ? commonIdentifiersItemRenderer(urlCategoriesSelectionModel.selectedValues.first)
          : "";

  String get urlListsSelectLabel =>
      urlListsSelectionModel.selectedValues.length > 0
          ? commonIdentifiersItemRenderer(urlListsSelectionModel.selectedValues.first)
          : "";

  String get quotasSelectLabel =>
      quotasSelectionModel.selectedValues.length > 0
          ? commonIdentifiersItemRenderer(quotasSelectionModel.selectedValues.first)
          : "";

  String defaultEumForRuleType;
  String defaultEumCallOutText;
  String selectedApplicationName;
  String url;
  String destination;
  String fileExtension;
  String userAgent;
  String primaryState = ButtonComponent.NORMAL_STATE;
  String validationMessage = "";
  String applicationProfileRequiredMessage = "";
  String dlpProfileRequiredMessage = "";
  String destinationRequiredMessage = "";
  String fileSizeRequiredMessage = "";
  String urlRequiredMessage = "";
  String urlListRequiredMessage = "";
  String urlCategoryRequiredMessage = "";
  String fileExtensionRequiredMessage = "";
  String fileExtensionGroupRequiredMessage = "";
  String fileTypeRequiredMessage = "";
  String applicationNameRequiredMessage = "";
  String applicationItemRequiredMessage = "";
  String userAgentRequiredMessage = "";
  String quotaProfileRequiredMessage = "";
  String actionRequiredMessage = "";
  String comment = "";

  PolicyIdentityRule selectedRule = null;
  bool dialogVisible = false;
  Action selectedAction;
  ChangeAction operation;
  int selectedRuleIndex;

  SelectionModel ruleTypesSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel applicationProfilesSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel applicationNamesSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel applicationItemsSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel dlpProfilesSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel fileExtensionGroupsSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel fileTypesSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel urlCategoriesSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel urlListsSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel quotasSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel actionsSelectionModel = new SelectionModel.withList(allowMulti: false, selectedValues: []);
  SelectionModel timeFrameIdentifiersSelectionModel = new SelectionModel.withList(
      allowMulti: false, selectedValues: []);
  SelectionModel eumOptionalIdentifiersSelectionModel = new SelectionModel.withList(
      allowMulti: false, selectedValues: []);


  ItemRenderer<RuleTypes> ruleTypesItemRenderer =
  new CachingItemRenderer<RuleTypes>((ruleType) => RuleUtil.getRuleTypesDisplayString(ruleType));

  ItemRenderer<Action> actionsItemRenderer =
  new CachingItemRenderer<Action>((action) => RuleUtil.getActionDisplayString(action));

  ItemRenderer<Identifier> commonIdentifiersItemRenderer = new CachingItemRenderer<Identifier>
    ((identifier) => "${identifier.name}");

  List<RelativePosition> get alphaPositions =>
      const [
        RelativePosition.AdjacentBottomLeft,
        RelativePosition.AdjacentBottomRight,
        RelativePosition.AdjacentTopLeft,
        RelativePosition.AdjacentTopRight
      ];
  List<EndUserMessage> defaultEndUserMessages = [];
  List dlpProfiles = [];
  List fileTypes = [];
  List fileExtensionGroups = [];
  List timeFrameIdentifiers = [];
  List eumOptionalIdentifiers = [];
  List appProfileIdentifiers = [];
  List urlListIdentifiers = [];
  List urlCategories = [];
  List urlCategoryIdentifiers = [];
  List quotasIdentifiers = [];
  List applicationNames = [];
  List applicationItems = [];
  List actionsProvider = [];

  Map allowedActions = {};
  Map<RuleTypes, Identifier> ruleTypesDefaultEums;
  Map appControlItemsMap = {};

  RuleTypes selectedRuleType;
  int fileSize;

  PopupSizeProvider callOutPopupSizeProvider = new FixedPopupSizeProvider(200, 200);

  PolicyIdentityRule newPolicyIdentityRule;
  var oldRule;

  validateUrl(String value) {
    return getUrlValidationMessage(value);
  }

  onAddRule(DatagridAddEvent addEvent) {
    _log.debug("onAddRule addEvent $addEvent");
    dialogTitle = interpret(SwgConfigLocaleInfo.ADD_RULE);
    newPolicyIdentityRule = new PolicyIdentityRule();
    newPolicyIdentityRule.id = TempIdGenerator.getId();
    dialogVisible = true;
    operation = ChangeAction.ADD;
    rulesForm.reset();
  }

  onEditRule(DatagridEditEvent editEvent) {
    _log.debug("onEditRule editEvent $editEvent");
    dialogTitle = interpret(SwgConfigLocaleInfo.EDIT_RULE);
    dialogVisible = true;
    newPolicyIdentityRule = selectedRule.clone();
    operation = ChangeAction.EDIT;
    resetMembers();
    applyProperties(newPolicyIdentityRule);
  }

  applyProperties(PolicyIdentityRule policyRule) {
    applyRuleProperties(policyRule.rule);
    applyBehaviorProperties(policyRule.behavior);
  }

  applyRuleProperties(Rule rule) {
    if (rule != null) {
      comment = rule.comment;
      ruleTypesSelectionModel.select(rule.type);
      switch (rule.type) {
        case RuleTypes.APP_CONTROL_PROFILE:
          AppControlProfileRule appControlProfileRule = rule as AppControlProfileRule;
          applicationProfilesSelectionModel.select(appControlProfileRule.appControlProfileIdentifier);
          break;
        case RuleTypes.CONTENT_SIZE:
          ContentSizeRule contentSizeRule = rule as ContentSizeRule;
          fileSize = contentSizeRule.fileSize;
          break;
        case RuleTypes.CUSTOM_APP_CONTROL:
          CustomAppControlRule customAppControlRule = rule as CustomAppControlRule;
          applicationNamesSelectionModel.select(customAppControlRule.appControlItem.applicationName);
          applicationItemsSelectionModel.select(customAppControlRule.appControlItem.itemName);
          break;
        case RuleTypes.CUSTOM_URL:
          CustomUrlRule customUrlRule = rule as CustomUrlRule;
          url = customUrlRule.urlItem.url;
          break;
        case RuleTypes.DLP:
          DlpRule dlpRule = rule as DlpRule;
          dlpProfilesSelectionModel.select(dlpRule.dlpProfileName);
          destination = dlpRule.destination;
          break;
        case RuleTypes.FILE_EXTENSION:
          FileExtensionRule fileExtensionRule = rule as FileExtensionRule;
          fileExtension = fileExtensionRule.extension;
          break;
        case RuleTypes.FILE_EXTENSION_GROUP:
          FileExtensionGroupRule fileExtensionGroupRule = rule as FileExtensionGroupRule;
          fileExtensionGroupsSelectionModel.select(fileExtensionGroupRule.fileExtensionGroupName);
          break;
        case RuleTypes.FILE_TYPE:
          FileTypeRule fileTypeRule = rule as FileTypeRule;
          fileTypesSelectionModel.select(fileTypeRule.fileType);
          break;
        case RuleTypes.URL_CATEGORY:
          UrlCategoryRule urlCategoryRule = rule as UrlCategoryRule;
          urlCategoriesSelectionModel.select(urlCategoryRule.categoryIdentifier);
          break;
        case RuleTypes.URL_LIST:
          UrlListRule urlListRule = rule as UrlListRule;
          urlListsSelectionModel.select(urlListRule.urlListIdentifier);
          break;
        case RuleTypes.USER_AGENT:
          UserAgentRule userAgentRule = rule as UserAgentRule;
          userAgent = userAgentRule.userAgent;
          break;
        case RuleTypes.QUOTA_PROFILE:
          QuotaRule quotaRule = rule as QuotaRule;
          quotasSelectionModel.select(quotaRule.quotaIdentifier);
          break;
      }
    }
  }

  applyBehaviorProperties(RuleBehavior behavior) {
    if (behavior != null) {
      if (behavior.action != null) {
        actionsSelectionModel.select(behavior.action);
      }
      if (behavior.timeframeIdentifier != null) {
        timeFrameIdentifiersSelectionModel.select(behavior.timeframeIdentifier);
      }

      if (behavior.endUserMessageIdentifier != null) {
        eumOptionalIdentifiersSelectionModel.select(behavior.endUserMessageIdentifier);
      }
    }
  }

  resetMembers() {
    ruleTypesSelectionModel.clear();
    applicationProfilesSelectionModel.clear();
    applicationNamesSelectionModel.clear();
    applicationItemsSelectionModel.clear();
    dlpProfilesSelectionModel.clear();
    urlListsSelectionModel.clear();
    urlCategoriesSelectionModel.clear();
    quotasSelectionModel.clear();
    actionsSelectionModel.clear();
    timeFrameIdentifiersSelectionModel.clear();
    eumOptionalIdentifiersSelectionModel.clear();
    defaultEumForRuleType = null;
    selectedRuleType = null;
    selectedAction = null;
    selectedApplicationName = null;
    comment = null;
    fileSize = null;
    url = null;
    destination = null;
    fileExtension = null;
    primaryState = ButtonComponent.NORMAL_STATE;
    validationMessage = "";
  }


  onDeleteRule(DatagridDeleteEvent deleteEvent) {
    _log.debug("onDeleteRule deleteEvent $deleteEvent");
    var deleteItem = deleteEvent.selectedItem;
    operation = ChangeAction.DELETE;
    rules.removeWhere((rule) => rule.id == deleteItem.id);
    rules = new List.from(rules);
  }

  onSelectRule(dynamic ruleItem) {
    _log.debug("onSelectItem ruleItem $ruleItem");
    selectedRule = ruleItem;
  }

  onSelectRuleIndex(int selectedItemIndex) {
    selectedRuleIndex = selectedItemIndex;
  }

  dialogOpen(DialogEvent dialogEvent) {
    _log.debug("dialogOpen selectedRule $selectedRule");
  }

  dialogClose(DialogEvent dialogEvent) {
    _log.debug("dialogClose selectedRule $selectedRule");
  }

  onDialogSubmit(DialogEvent dialogEvent) async {
    _log.debug("onDialogSubmit start dialogEvent $dialogEvent");
    primaryState = ButtonComponent.NORMAL_STATE;
    validationMessage = "";
    bool valid = rulesForm.validate();
    if (!valid) {
      primaryState = ButtonComponent.ERROR_STATE;
      validationMessage = interpret(SwgConfigLocaleInfo.RULE_FORM_ERROR_MESSAGE);
      return;
    } else {
      if (newPolicyIdentityRule.rule == null && selectedRuleType != null) {
        newPolicyIdentityRule.rule = new Rule.fromType(selectedRuleType);
        newPolicyIdentityRule.rule.id = TempIdGenerator.getId();
        newPolicyIdentityRule.behavior = new RuleBehavior();
      }
      applyRuleFieldsFromForm(newPolicyIdentityRule.rule);
      applyBehaviorFieldsFromForm(newPolicyIdentityRule.behavior);
      validationMessage = await validateRuleExists();
      if (validationMessage.isNotEmpty) {
        primaryState = ButtonComponent.ERROR_STATE;
        return;
      }
    }
    if (operation == ChangeAction.EDIT) {
      rules.removeWhere((PolicyIdentityRule piRule)  => piRule.id == newPolicyIdentityRule.id);
      rules.add(newPolicyIdentityRule);
      //rulesGrid.updateRow(selectedRuleIndex, newPolicyIdentityRule);
    } else {
      rules.add(newPolicyIdentityRule);
    }
    List rulesCopy = new List.from(rules);
    rules = rulesCopy;
    dialogVisible = false;
    _log.debug("onDialogSubmit end dialogEvent $dialogEvent");
  }

  getSelectedItem(SelectionModel selectionModel) {
    return selectionModel.selectedValues.length > 0
        ? selectionModel.selectedValues.first : null;
  }

  applyRuleFieldsFromForm(Rule rule) {
    if (rule is AppControlProfileRule) {
      rule.appControlProfileIdentifier = getSelectedItem(applicationProfilesSelectionModel);
    } else if (rule is ContentSizeRule) {
      rule.fileSize = fileSize;
    } else if (rule is CustomAppControlRule) {
      rule.appControlItem = new AppControlItem()
        ..applicationName = getSelectedItem(applicationNamesSelectionModel)
        ..itemName = getSelectedItem(applicationItemsSelectionModel);
    } else if (rule is CustomUrlRule) {
      UrlItem urlItem = new UrlItem()
        ..url = idnaUrlEncoder(url);
      rule.urlItem = urlItem;
    } else if (rule is DlpRule) {
      rule.dlpProfileName = getSelectedItem(dlpProfilesSelectionModel);
      rule.destination = destination;
    } else if (rule is FileExtensionGroupRule) {
      rule.fileExtensionGroupName = getSelectedItem(fileExtensionGroupsSelectionModel);
    } else if (rule is FileExtensionRule) {
      rule.extension = fileExtension.toUpperCase();
    } else if (rule is FileTypeRule) {
      rule.fileType = getSelectedItem(fileTypesSelectionModel);
    } else if (rule is UrlCategoryRule) {
      rule.categoryIdentifier = getSelectedItem(urlCategoriesSelectionModel);
    } else if (rule is UrlListRule) {
      rule.urlListIdentifier = getSelectedItem(urlListsSelectionModel);
    } else if (rule is UserAgentRule) {
      rule.userAgent = userAgent;
    } else if (rule is QuotaRule) {
      rule.quotaIdentifier = getSelectedItem(quotasSelectionModel);;
    }
    rule.comment = comment;
  }

  applyBehaviorFieldsFromForm(RuleBehavior behavior) {
    behavior.action = getSelectedItem(actionsSelectionModel);
    behavior.timeframeIdentifier = getSelectedItem(timeFrameIdentifiersSelectionModel);
    behavior.endUserMessageIdentifier = getSelectedItem(eumOptionalIdentifiersSelectionModel);
    behavior.comment = comment;
  }

  Future<String> validateRuleExists() async {
    _log.debug("validateRuleExists");
    String msg = "";
    PolicyIdentityRule existingRule = rules.firstWhere((PolicyIdentityRule policyIdentityRule) =>
    policyIdentityRule.rule.identity(newPolicyIdentityRule.rule) &&
        policyIdentityRule.id != newPolicyIdentityRule.id, orElse: () => null) ;
    if (existingRule != null) {
      Identifier currentTimeFrameIdentifier = newPolicyIdentityRule.behavior.timeframeIdentifier;
      Identifier existingTimeFrameIdentifier = existingRule.behavior.timeframeIdentifier;
      if ((currentTimeFrameIdentifier == null &&
          existingTimeFrameIdentifier != null)) {
        msg = interpret(SwgConfigLocaleInfo.TIME_FRAME_OVERLAPS_MSG1, [existingTimeFrameIdentifier.name]);
        _log.debug("validateRuleExists msg $msg");
      } else if ((currentTimeFrameIdentifier != null &&
          existingTimeFrameIdentifier == null)) {
        msg = interpret(SwgConfigLocaleInfo.TIME_FRAME_OVERLAPS_MSG2, [currentTimeFrameIdentifier.name]);
        _log.debug("validateRuleExists msg $msg");
      } else if ((currentTimeFrameIdentifier == null && existingTimeFrameIdentifier == null) ||
          (currentTimeFrameIdentifier == existingTimeFrameIdentifier)) {
        msg = interpret(SwgConfigLocaleInfo.TIME_FRAME_OVERLAPS_MSG3);
        _log.debug("validateRuleExists msg $msg");
      } else {

        List<TimeFrame> timeFrames = await _identifierCache.getAllTimeFrames();
        TimeFrame currentTimeFrame = getTimeFrame(timeFrames, currentTimeFrameIdentifier);
        TimeFrame existingTimeFrame = getTimeFrame(timeFrames, existingTimeFrameIdentifier);
        if (currentTimeFrame != null && existingTimeFrame != null) {
          if (currentTimeFrame.overlaps(existingTimeFrame)) {
            msg = interpret(SwgConfigLocaleInfo.TIME_FRAME_OVERLAPS_MSG4, [currentTimeFrameIdentifier.name, existingTimeFrame.name]);
            _log.debug("validateRuleExists overlaps msg $msg");
          }
        }
      }
    }
    return msg;
  }

  TimeFrame getTimeFrame(List<TimeFrame> timeFrames, Identifier timeFrameIdentifier) {
    TimeFrame timeFrame = null;
    // these change set code needs to be changed as per new change set design here.
    // current code is
    List<Change> timeFrameChanges = new ChangeSet().getChanges().where((change) => (change.type == TimeFrame.type));
    Change change = timeFrameChanges?.firstWhere((change) => (change.modelVO as TimeFrame).id == timeFrameIdentifier.id, orElse: () => null);
    timeFrame = (change != null) ? change.modelVO : null;
    if (timeFrame == null) {
      timeFrame = timeFrames.firstWhere((timeFrame) => timeFrame.id == timeFrameIdentifier.id,
          orElse: () => null);
    }
    return timeFrame;
  }

  onDialogCancel(DialogEvent dialogEvent) {
    _log.debug("onDialogCancel dialogEvent $dialogEvent");
  }

  Map ruleSummaryValueFunction(dynamic dataValue, String dataField, int index, dynamic item) {
    _log.debug("dataValue $dataValue dataField, $dataField index $index item $item");
    return _toMap(RuleUtil.getRuleSummary(dataValue));
  }

  Map actionValueFunction(dynamic dataValue, String dataField, int index, dynamic item) {
    _log.debug("dataValue $dataValue dataField, $dataField index $index item $item");
    return _toMap(RuleUtil.getActionDisplayString(dataValue.action));
  }

  Map timeFrameValueFunction(dynamic dataValue, String dataField, int index, dynamic item) {
    _log.debug("dataValue $dataValue dataField, $dataField index $index item $item");
    return _toMap(RuleUtil.getTimeFrameDisplayString(dataValue));
  }

  Map eumValueFunction(dynamic dataValue, String dataField, int index, dynamic item) {
    _log.debug("dataValue $dataValue dataField, $dataField index $index item $item");
    return _toMap(RuleUtil.getEUMDisplayString(dataValue));
  }

  ///
  /// Component View Variables
  ///

  @ViewChild('rulesDialog')
  DialogComponent rulesDialog;

  @ViewChild('rulesForm')
  FormComponent rulesForm;

  @ViewChild('rulesGrid')
  DatagridComponent rulesGrid;

  //
  // Lifecycle Methods
  //

  @override
  onNgInit() {
    _initialize();
  }

  onLocaleLoaded() {
    applicationProfileRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.APPLICATION_PROFILE)]);
    _log.debug("applicationProfileRequiredMessage $applicationProfileRequiredMessage");
    dlpProfileRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.PROFILE_NAME)]);
    destinationRequiredMessage = interpret(SwgConfigLocaleInfo.INPUT_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.DESTINATION)]);
    fileSizeRequiredMessage = interpret(SwgConfigLocaleInfo.INPUT_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.FILE_SIZE)]);
    urlRequiredMessage = interpret(SwgConfigLocaleInfo.INPUT_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.URL)]);
    urlListRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.URL_LIST)]);
    urlCategoryRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.URL_CATEGORY)]);
    fileExtensionRequiredMessage = interpret(SwgConfigLocaleInfo.INPUT_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.FILE_EXTENSION)]);
    fileExtensionGroupRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.FILE_EXTENSION_GROUP)]);
    fileTypeRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.FILE_TYPE)]);
    applicationNameRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.APPLICATION_NAME)]);
    applicationItemRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.APPLICATION_ITEM)]);
    quotaProfileRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.QUOTA)]);
    userAgentRequiredMessage = interpret(SwgConfigLocaleInfo.INPUT_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.USER_AGENT)]);
    actionRequiredMessage = interpret(SwgConfigLocaleInfo.SELECTION_REQUIRED_ERROR_MESSAGE,
        [interpret(SwgConfigLocaleInfo.ACTION)]);
    _log.debug("userAgentRequiredMessage $userAgentRequiredMessage");
  }


  //
  // Private Members
  //

  _toMap(dynamic value) {
    Map retValue = new Map();
    retValue['value'] = value;
    retValue['tooltip'] = value;
    return retValue;
  }

  _initialize() async {
    await _loadData();
    ruleTypesSelectionModel.selectionChanges.listen((List<SelectionChangeRecord> values) {
      _onRuleTypeChange();
    });

    applicationNamesSelectionModel.selectionChanges.listen((List<SelectionChangeRecord> values) {
      _onApplicationNameChange();
    });

    actionsSelectionModel.selectionChanges.listen((List<SelectionChangeRecord> values) {
      _onActionChange();
    });
  }

  _onRuleTypeChange() {
    selectedRuleType =
    ruleTypesSelectionModel.selectedValues.length > 0 ? ruleTypesSelectionModel.selectedValues.first : null;
    if (selectedRuleType != null) {
      actionsProvider = new List.from(allowedActions[selectedRuleType]);
    }

    if (selectedRuleType == RuleTypes.CUSTOM_APP_CONTROL) {
      applicationNames = new List.from(appControlItemsMap.keys);
    }

    Identifier eumIdentifier = ruleTypesDefaultEums[selectedRuleType];
    defaultEumForRuleType = (eumIdentifier != null) ? eumIdentifier.name : "";
    defaultEumCallOutText = defaultEndUserMessages
        .firstWhere((eum) => eum.id == eumIdentifier?.id, orElse: () => null)
        ?.customText ?? "";
    defaultEumCallOutText.replaceAll("%new_line%", "\n");
  }

  _onApplicationNameChange() {
    selectedApplicationName = applicationNamesSelectionModel.selectedValues.length > 0 ?
    applicationNamesSelectionModel.selectedValues.first : null;
    applicationItems = new List.from(appControlItemsMap[selectedApplicationName]);
  }

  _onActionChange() {
    selectedAction =
    actionsSelectionModel.selectedValues.length > 0 ? actionsSelectionModel.selectedValues.first : null;
    eumOptionalIdentifiersSelectionModel.clear();
    if (selectedRule != null && selectedRule.behavior.endUserMessageIdentifier != null) {
      eumOptionalIdentifiersSelectionModel.select(selectedRule.behavior.endUserMessageIdentifier);
    }
  }

  _loadData() {
    _loadAllowedActions();
    _loadDefaultEndUserMessages();
    _loadRuleTypesDefaultEums();
    _loadDlpProfiles();
    _loadFileTypes();
    _loadFileExtensionGroups();
    _loadAppControlItemsMap();
    _loadTimeFrameIdentifiers();
    _loadEumOptionalIdentifiers();
    _loadAppProfileIdentifiers();
    _loadUrlListIdentifiers();
    _loadUrlCategories();
    _loadQuotas();
  }

  _loadAllowedActions() async {
    allowedActions = new Map.from(await _identifierCache.getAllowedActions());
  }

  _loadDefaultEndUserMessages() async {
    defaultEndUserMessages = new List.from(await _identifierCache.getDefaultEndUserMessages());
  }

  _loadRuleTypesDefaultEums() async {
    ruleTypesDefaultEums = new Map.from(await _identifierCache.getDefaultEumMap());
  }

  _loadDlpProfiles() async {
    List cpyDlpProfiles = new List.from(await _identifierCache.getDLPProfiles());
    cpyDlpProfiles.sort();
    dlpProfiles = cpyDlpProfiles;
  }

  _loadFileTypes() async {
    List cpyFileTypes = new List.from(await _identifierCache.getFileTypes());
    cpyFileTypes.sort();
    fileTypes = cpyFileTypes;
  }

  _loadFileExtensionGroups() async {
    List cpyExtensionGroups = new List.from(await _identifierCache.getFileExtensionGroups());
    cpyExtensionGroups.sort();
    fileExtensionGroups = cpyExtensionGroups;
  }


  _loadAppControlItemsMap() async {
    List appControlItemsList = await _identifierCache.getApplicationControlItems();
    Map copyMap = {};
    if (appControlItemsList != null) {
      for (AppControlItem item in appControlItemsList) {
        if (copyMap[item.applicationName] == null) {
          copyMap[item.applicationName] = [item.itemName];
        } else {
          copyMap[item.applicationName].add(item.itemName);
        }
      }
      appControlItemsMap = copyMap;
    }
  }

  _loadTimeFrameIdentifiers() async {
    List<Identifier> cachedFrames = await _identifierCache.getItemsOfType(IdentifierType.TIME_FRAME);
    cachedFrames.sort((a, b) => a.name.toLowerCase().compareTo(b.name.toLowerCase()));
    timeFrameIdentifiers = cachedFrames;
  }

  _loadEumOptionalIdentifiers() async {
    List<Identifier> cachedEums = await _identifierCache.getItemsOfType(IdentifierType.EUM_CUSTOMER);
    cachedEums.sort((a, b) => a.name.toLowerCase().compareTo(b.name.toLowerCase()));
    eumOptionalIdentifiers = cachedEums;
  }

  _loadAppProfileIdentifiers() async {
    List<Identifier> cachedProfiles = await _identifierCache.getItemsOfType(IdentifierType.APP_PROFILE);
    cachedProfiles.sort((a, b) => a.name.toLowerCase().compareTo(b.name.toLowerCase()));
    appProfileIdentifiers = cachedProfiles;
  }

  _loadUrlListIdentifiers() async {
    List<Identifier> cachedLists = await _identifierCache.getItemsOfType(IdentifierType.URL_LIST);
    cachedLists.sort((a, b) => a.name.toLowerCase().compareTo(b.name.toLowerCase()));
    urlListIdentifiers = cachedLists;
  }

  _loadUrlCategories() async {
    List categoryIdentifiers = new List.from(await _identifierCache.getCustomerUrlCategoryIdentifiers(CustomerModelLocator.customer.id));
    categoryIdentifiers.sort();
    urlCategoryIdentifiers = categoryIdentifiers;
  }

  _loadQuotas() async {
    List<Identifier> cachedLists = await _identifierCache.getItemsOfType(IdentifierType.QUOTA);
    cachedLists.sort((a, b) => a.name.toLowerCase().compareTo(b.name.toLowerCase()));
    quotasIdentifiers = cachedLists;
  }
}